use serde::{Deserialize, Serialize};

#[derive(Clone, Serialize, Deserialize, Debug)]
pub struct PostMetadata {
    pub image_path: String,
    pub title: String,
    pub date: String,
    pub description: String,
    pub project_link: String,
}

#[derive(Clone, Serialize, Deserialize, Debug)]
pub struct Post {
    pub metadata: PostMetadata,
    pub content: String,
}

cfg_if::cfg_if! {
    if #[cfg(feature = "ssr")] {
        impl Post {
            fn from_content(content: String) -> Option<Self> {
                use gray_matter::{Matter, engine::YAML};
                let matter = Matter::<YAML>::new();

                let post_data = matter
                    .parse_with_struct::<PostMetadata>(&content)?;

                let metadata = post_data.data;
                let content = post_data.content;

                use pulldown_cmark::{Parser, Options, html};
                let parser = Parser::new_ext(&content, Options::all());
                let mut html_output = String::new();
                html::push_html(&mut html_output, parser);

                Some(Self {
                    metadata,
                    content: html_output,
                })
            }

            fn from_path(path: &std::path::Path) -> Option<Self> {
                let content = std::fs::read_to_string(path);
                let content = match content {
                    Ok(content) => content,
                    Err(e) => {
                        eprintln!("{:?}", e);
                        return None;
                    }
                };

                Self::from_content(content)
            }

            pub fn get_all(folder: &str) -> Result<Vec<Self>, String> {
                use std::{path::Path, fs::read_dir};

                let mut posts: Vec<Self> = Vec::new();

                let folder_path = Path::new("markdowns").join(folder);
                let paths = read_dir(folder_path).map_err(|e| e.to_string())?;

                for path_result in paths {
                    let path = match path_result {
                        Ok(path) => path.path(),
                        Err(e) => {
                            eprintln!("{:?}", e);
                            continue;
                        }
                    };

                    if let Some(post) = Self::from_path(&path) {
                        posts.push(post);
                    }
                }

                Ok(posts)
            }
        }
    }
}

